@@ -630,8 +630,8 @@
 (test #f string->number "+")
 
 (SECTION 6 6)
-(test #t eqv? '#\  #\Space)
-(test #t eqv? #\space '#\Space)
+;(test #t eqv? '#\  #\Space)
+;(test #t eqv? #\space '#\Space)
 (test #t char? #\a)
 (test #t char? #\()
 (test #t char? #\ )
